from ..object_classes.flat_faces_object import FlatFacesObject
import numpy as np


class Vector(FlatFacesObject):
    def __init__(
        self,
        start_point=np.array([0.0, 0.0, 0.0]),
        rotation=np.array([0.0, 0.0, 0.0]),
        length=1,
        color=(0, 0, 0),
        thickness=0.02,
    ):
        """
        Vector object
        :param start_point: the start point of the vector
        :param rotation: the rotation of the vector
        :param length: the length of the vector
        :param color: the color of the vector
        :param thickness: the thickness of the vector
        """

        vertices = []
        faces = []
        pi2 = np.pi * 2

        for i in range(max(round(length * 4), 1)):
            vertices.append(
                np.array(
                    [np.cos(pi2 / 3) * thickness, np.sin(pi2 / 3) * thickness, i / 4]
                )
            )
            vertices.append(
                np.array(
                    [
                        np.cos(pi2 * 2 / 3) * thickness,
                        np.sin(pi2 * 2 / 3) * thickness,
                        i / 4,
                    ]
                )
            )
            vertices.append(
                np.array([np.cos(pi2) * thickness, np.sin(pi2) * thickness, i / 4])
            )

        vertices.append(
            np.array(
                [
                    np.cos(pi2 / 3) * thickness,
                    np.sin(pi2 / 3) * thickness,
                    length - 0.03,
                ]
            )
        )
        vertices.append(
            np.array(
                [
                    np.cos(pi2 * 2 / 3) * thickness,
                    np.sin(pi2 * 2 / 3) * thickness,
                    length - 0.03,
                ]
            )
        )
        vertices.append(
            np.array([np.cos(pi2) * thickness, np.sin(pi2) * thickness, length - 0.03])
        )

        # make faces between the vertices so that outside of the vector is filled in
        # ex: (0, 1, 3, 4) makes a face between the two ends of the vector
        for i in range(0, len(vertices) - 3, 3):
            for j in range(3):
                if j == 2:
                    faces.append(
                        (
                            [i + 2, i, i + 3, i + 5],
                            color,
                        )
                    )
                else:
                    faces.append(
                        (
                            [i + j, i + j + 1, i + j + 4, i + j + 3],
                            color,
                        )
                    )

        # make arrow head
        for i in range(4):
            vertices.append(
                np.array(
                    [
                        np.cos(pi2 * (i + 1) / 4) * thickness * 2,
                        np.sin(pi2 * (i + 1) / 4) * thickness * 2,
                        length - 0.03,
                    ],
                )
            )
        vertices.append(np.array([0, 0, length], dtype=float))

        faces.append(([len(vertices) - 5, len(vertices) - 4, len(vertices) - 1], color))
        faces.append(
            (
                [len(vertices) - 4, len(vertices) - 3, len(vertices) - 1],
                color,
            )
        )
        faces.append(
            (
                [len(vertices) - 3, len(vertices) - 2, len(vertices) - 1],
                color,
            )
        )
        faces.append(
            (
                [len(vertices) - 2, len(vertices) - 5, len(vertices) - 1],
                color,
            )
        )

        super().__init__(
            vertices=vertices,
            faces=faces,
            color=color,
            compile_verts=True,
            position=start_point,
        )



import numpy as np
from numba import njit


@njit(fastmath=True)
def project_point(point, offset_array, focal_length, screen_size, fov_side, fov_top):
    if point[1] <= 0:
        return None, 1

    # if point is out of view, return None
    top_angle = np.arctan2(point[2], point[1])
    side_angle = np.arctan2(point[0], point[1])

    if abs(top_angle) > fov_top or abs(side_angle) > fov_side:
        return None, 1

    focal_length_divided_by_y = focal_length / point[1]

    projected_point = np.array(
        [
            point[0] * focal_length_divided_by_y,
            -point[2] * focal_length_divided_by_y,
        ]
    )

    projected_point += offset_array

    # if the point is more than 2000 pixels off the edge of the screen, return None
    if (
        abs(projected_point[0]) > screen_size[0] + 2000
        or abs(projected_point[1]) > screen_size[1] + 2000
    ):
        return None, 1

    point_size = 1 / np.linalg.norm(point)

    return projected_point, point_size
